#include "Win32_WindowFileInfo.h"
#include "Win32_StaticControl.h"
#include "Win32_PushButton.h"
#include "FileImage.h"
#include "c64file.h"
#include "resource.h"
#include "stringops.h"
#include "window.h"
#include "debug.h"

#include "global.h"
#include "md5.h"
#include "sha1.h"

static WindowFileInfo *HelpWin = 0;

static LRESULT CALLBACK HelpCallBack(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	if (HelpWin) return HelpWin->ProcessMessage(hwnd, msg, wParam, lParam);
	return DefWindowProc(hwnd, msg, wParam, lParam);
}

#include <stdio.h>

WindowFileInfo::WindowFileInfo(HINSTANCE hInstance, CbmWindow *mainwin, unsigned long index, SP<class FileImage> img)
{
	MainWin = mainwin;
	Index = index;
	Image = img;
	unsigned char *data;
	unsigned long size;
	int adress;
	char hexbuffer[64];
	unsigned char md5buffer[16];
	unsigned char sha1buffer[20];

	data = 0;
	size = 0;
	adress = -1;

	File = img->GetFile(index);
	if (File != 0) {
		data = File->GetData();
		size = File->GetSize();
		if ((size > 1) && (data != 0)) {
			adress  = (int)(data[1]);
			adress <<= 8;
			adress |= (int)(data[0]);
		}
	}

	md5calc((Uint8 *)data, (Uint32)size, (Uint8 *)md5buffer);
	sha1calc((Uint8 *)data, (Uint32)size, (Uint8 *)sha1buffer);

	if (File != 0) delete File;

	TextLeft = "Load adress:\nFile size:\n\nMD5:\nSHA-1:\n";

	TextRight = new StringStream();
	hexbuffer[0] = '$';
	printhexbyte(&hexbuffer[1], adress >> 8);
	printhexbyte(&hexbuffer[3], adress & 0xFF);
	TextRight = TextRight->Append(hexbuffer);
	TextRight = TextRight->Append("\n");

	printint(hexbuffer, size);
	TextRight = TextRight->Append(hexbuffer);
	TextRight = TextRight->Append("\n\n");

	printhexarray(hexbuffer, md5buffer, 16);
	TextRight = TextRight->Append(hexbuffer);
	TextRight = TextRight->Append("\n");

	printhexarray(hexbuffer, sha1buffer, 20);
	TextRight = TextRight->Append(hexbuffer);
	TextRight = TextRight->Append("\n");

	Instance = hInstance;
	TextLeftControl = 0;
	TextRightControl = 0;
	ButtonOk = 0;
}

WindowFileInfo::~WindowFileInfo(void)
{
	CloseControls();
}

void WindowFileInfo::CloseControls(void)
{
	if (TextLeftControl) delete TextLeftControl;
	if (TextRightControl) delete TextRightControl;
	if (ButtonOk) delete ButtonOk;
	TextLeftControl = 0;
	TextRightControl = 0;
	ButtonOk = 0;
	TextLeft = 0;
	TextRight = 0;
}

void WindowFileInfo::Open(HWND parent)
{
	WNDCLASSEX wc;
	HWND Handle;
	long x, y;
	char clsname[]="Arc64_WindowFileInfo";

	if (HelpWin) return;
	HelpWin = this;

	wc.cbSize		= sizeof(WNDCLASSEX);
	wc.style		= 0;
	wc.lpfnWndProc	= HelpCallBack;
	wc.cbClsExtra	= 0;
	wc.cbWndExtra	= 0;
	wc.hInstance	= Instance;
	wc.hIcon		= LoadIcon(Instance, MAKEINTRESOURCE(IDI_ICON2));
	wc.hIconSm		= LoadIcon(Instance, MAKEINTRESOURCE(IDI_ICON1));
	wc.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = GetSysColorBrush(COLOR_3DFACE);
	wc.lpszMenuName	= NULL;
	wc.lpszClassName= clsname;

	if (!RegisterClassEx(&wc)) {
//		MessageBox(NULL, "Could not register window!", "Error!",
	//		MB_ICONEXCLAMATION | MB_OK);
		//return;
	}

	x = 720;
	y = 200;

	Handle = CreateWindowEx(0,
		clsname, "File Info",
		WS_OVERLAPPEDWINDOW & (~(WS_MINIMIZEBOX | WS_MAXIMIZEBOX)),
		CW_USEDEFAULT, CW_USEDEFAULT, x, y,
		parent, NULL, Instance, NULL);

	if (Handle == NULL) {
		HelpWin = 0;
		MessageBox(NULL, "Could not create window!", "Error!",
			MB_ICONEXCLAMATION | MB_OK);
		return;
	}

	ShowWindow(Handle, SW_SHOW);
	UpdateWindow(Handle);
}

LRESULT WindowFileInfo::ProcessMessage(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	RECT clientrect;

	switch (msg)
	{
	default:
		break;

	case WM_COMMAND:
	case WM_NOTIFY:
		if ((ButtonOk) && (HIWORD(wParam) == BN_CLICKED)) {
			if (ButtonOk->IsControl((HWND)lParam)) {
				CloseControls();
				DestroyWindow(hwnd);
			}
		}
		return 0;

	case WM_KEYDOWN:
		switch (wParam)
		{
		default:
			break;
		case (VK_RETURN):
		case (VK_ESCAPE):
			CloseControls();
			DestroyWindow(hwnd);
			break;
		}
		break;

	case WM_CLOSE:
		CloseControls();
		DestroyWindow(hwnd);
		return 0;

	case WM_DESTROY:
		delete HelpWin;
		HelpWin = 0;
		break;

	case WM_CREATE:
		TextLeftControl = new StaticControl(hwnd, Instance, TextLeft);
		TextRightControl = new StaticControl(hwnd, Instance, TextRight->GetString());
		ButtonOk = new PushButton(hwnd, Instance, "Ok");
		return 0;

	case WM_ERASEBKGND:
		if (TextLeftControl) TextLeftControl->ExcludeFrom((HDC)wParam);
		if (TextRightControl) TextRightControl->ExcludeFrom((HDC)wParam);
		if (ButtonOk) ButtonOk->ExcludeFrom((HDC)wParam);
		break;

	case WM_PAINT:
		if (TextLeftControl) TextLeftControl->Update(hwnd);
		if (TextRightControl) TextRightControl->Update(hwnd);
		if (ButtonOk) ButtonOk->Update(hwnd);
		return 0;

	case WM_SIZE:
		GetClientRect(hwnd, &clientrect);

		if (TextLeftControl) TextLeftControl->SetPos(8, 8, 100, clientrect.bottom-38-16);
		if (TextRightControl) TextRightControl->SetPos(108, 8, clientrect.right-16-100, clientrect.bottom-38-16);
		if (ButtonOk) ButtonOk->SetPos(8, clientrect.bottom-38, 70, 30);
		return 0;

	case WM_GETMINMAXINFO:
		MINMAXINFO FAR *mmi;
		mmi = (MINMAXINFO FAR *) lParam;
		mmi->ptMinTrackSize.x = 480+GetSystemMetrics(SM_CXFRAME)*2;
		mmi->ptMinTrackSize.y = GetSystemMetrics(SM_CYFRAME)*2+GetSystemMetrics(SM_CYCAPTION)+160;
		mmi->ptMaxTrackSize.x = mmi->ptMinTrackSize.x;
		mmi->ptMaxTrackSize.y = mmi->ptMinTrackSize.y;
		return 0;
	}
	return DefWindowProc(hwnd, msg, wParam, lParam);
}

