/*
Copyright (c) 2014, John Selck
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include "global.h"

/**
* \brief	Typical initialization values for a lot of hash algorithms (MD5, SHA-1, SHA-2, RIPEMD...)
*/
const Uint32 initconst[5] =
{
	0x67452301,
	0xEFCDAB89,
	0x98BADCFE,
	0x10325476,
	0xC3D2E1F0
};

/**
* \brief	Typical padding values for a lot of hash algorithms (MD5, SHA-1, SHA-2, RIPEMD, Tiger2...)
*/
const Uint8 padding[128] =
{
	0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

Uint32 getLongBigEndian(const Uint8* data)
{
	Uint32 x;

	x  = data[0];
	x <<= 8;
	x |= data[1];
	x <<= 8;
	x |= data[2];
	x <<= 8;
	x |= data[3];

	return (x);
}

Uint32 getLongLittleEndian(const Uint8* data)
{
	Uint32 x;

	x  = data[3];
	x <<= 8;
	x |= data[2];
	x <<= 8;
	x |= data[1];
	x <<= 8;
	x |= data[0];

	return (x);
}

void setLongBigEndian(Uint32 x, Uint8* data)
{
	data[3] = (Uint8)(x & 0xFF);
	x >>= 8;
	data[2] = (Uint8)(x & 0xFF);
	x >>= 8;
	data[1] = (Uint8)(x & 0xFF);
	x >>= 8;
	data[0] = (Uint8)(x & 0xFF);
}

void setLongLittleEndian(Uint32 x, Uint8* data)
{
	data[0] = (Uint8)(x & 0xFF);
	x >>= 8;
	data[1] = (Uint8)(x & 0xFF);
	x >>= 8;
	data[2] = (Uint8)(x & 0xFF);
	x >>= 8;
	data[3] = (Uint8)(x & 0xFF);
}

Uint32 leftrotate(Uint32 x, Uint8 r)
{
	return ((x << r) | (x >> (32-r)));
}
