/*
Copyright (c) 2014, John Selck
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include "global.h"
#include "md5.h"
#include "md5core.h"

static Uint32 md5sum[4];
static Uint8 md5_chunk[64];
static Uint32 md5_size;

static void setLongLittleEndian(Uint32 x, Uint8* data)
{
	data[0] = (Uint8)(x & 0xFF);
	x >>= 8;
	data[1] = (Uint8)(x & 0xFF);
	x >>= 8;
	data[2] = (Uint8)(x & 0xFF);
	x >>= 8;
	data[3] = (Uint8)(x & 0xFF);
}

/**
* \brief	Initialize MD5 calculation
*/
void md5init(void)
{
	Sint16 i;

	for (i=0;i<4;i++) {
		md5sum[i] = initconst[i];
	}

	md5_size = 0;
}

/**
* \brief	Update MD5 hash with any data bytes.
*
* \param	data	Pointer to data
* \param	size	Number of bytes
*/
void md5update(const Uint8* data, Uint32 size)
{
	Sint16 k;
	Uint32 i, modsize;

	k = (Sint16)(md5_size & 0x3F);

	if ((k == 0) && (size >= 64)) {

		md5updatechunks(data, size >> 6, md5sum);

		// skip processed data in next loop
		modsize = size & 0x3F;
		size -= modsize;
		data += size;
		md5_size += size;
		size = modsize;
	}

	i = 0;
	while (i < size) {
		while ((k < 64) && (i < size)) {
			md5_chunk[k] = data[i];
			k++;
			i++;
		}

		if (k >= 64) {
			md5updatechunks(md5_chunk, 1, md5sum);
			k = 0;
		}
	}
	md5_size += size;
}

void md5finish(Uint8* sum)
{
	Uint32 m;
	Uint8 endsize[8];

	setLongLittleEndian((md5_size<< 3), &endsize[0]);
	setLongLittleEndian((md5_size>>29), &endsize[4]);

	m = 0x40 - ((md5_size+8) & 0x3F);
	md5update(padding, m);
	md5update(endsize, 8);

	for (m=0;m<4;m++) {
		setLongLittleEndian(md5sum[m], &sum[m<<2]);
	}
}

void md5calc(const Uint8* data, Uint32 size, Uint8* sum)
{
	md5init();
	md5update(data, size);
	md5finish(sum);
}
