/*
Copyright (c) 2014, John Selck
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include "global.h"
#include "sha1.h"
#include "sha1core.h"

static Uint32 sha1sum[5];
static Uint8 sha1_chunk[64];
static Uint32 sha1_size;

void sha1init(void)
{
	Sint16 i;

	for (i=0;i<5;i++) {
		sha1sum[i] = initconst[i];
	}

	sha1_size = 0;
}

void sha1update(const Uint8* data, Uint32 size)
{
	Sint16 k;
	Uint32 i, modsize;

	k = (Sint16)(sha1_size & 0x3F);

	if ((k == 0) && (size >= 64)) {

		sha1updatechunks(data, size >> 6, sha1sum);

		// skip processed data in next loop
		modsize = size & 0x3F;
		size -= modsize;
		data += size;
		sha1_size += size;
		size = modsize;
	}

	i = 0;
	while (i < size) {
		while ((k < 64) && (i < size)) {
			sha1_chunk[k] = data[i];
			k++;
			i++;
		}

		if (k >= 64) {
			sha1updatechunks(sha1_chunk, 1, sha1sum);
			k = 0;
		}
	}
	sha1_size += size;
}

void sha1finish(Uint8* sum)
{
	Uint32 m;
	Uint8 endsize[8];

	setLongBigEndian((sha1_size>>29), &endsize[0]);
	setLongBigEndian((sha1_size<< 3), &endsize[4]);

	m = 0x40 - ((sha1_size+8) & 0x3F);
	sha1update(padding, m);
	sha1update(endsize, 8);

	for (m=0;m<5;m++) {
		setLongBigEndian(sha1sum[m], &sum[m<<2]);
	}
}

void sha1calc(const Uint8* data, Uint32 size, Uint8* sum)
{
	sha1init();
	sha1update(data, size);
	sha1finish(sum);
}
