/*
Copyright (c) 2014, John Selck
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include "global.h"
#include "sha1core.h"

/**
* \brief	SHA-1
*
* \param	chunk	Pointer to 64 byte chunk
* \param	h	Pointer to current hash value
*/
static void sha1chunk(const Uint8* chunk, Uint32* h)
{
	Uint32 a, b, c, d, e, f, k, temp;
	Sint16 i;
	Uint32 w[80];

	for (i=0;i<16;i++) {
		w[i] = getLongBigEndian(&chunk[i<<2]);
	}

	for (i=16;i<80;i++) {
		w[i] = leftrotate((w[i-3] ^ w[i-8] ^ w[i-14] ^ w[i-16]), 1);
	}

	a = h[0];
	b = h[1];
	c = h[2];
	d = h[3];
	e = h[4];

	k = 0x5A827999;		// floor(sqrt(2)*2^30)
	for (i=0;i<20;i++) {

		f = d ^ (b & (c ^ d));
		e = leftrotate(a, 5) + f + e + k + w[i];
		b = leftrotate(b, 30);

		temp = e;
		e = d;
		d = c;
		c = b;
		b = a;
		a = temp;
	}

	k = 0x6ED9EBA1;		// floor(sqrt(3)*2^30)
	for (i=20;i<40;i++) {

		f = b ^ c ^ d;
		e = leftrotate(a, 5) + f + e + k + w[i];
		b = leftrotate(b, 30);

		temp = e;
		e = d;
		d = c;
		c = b;
		b = a;
		a = temp;
	}

	k = 0x8F1BBCDC;		// floor(sqrt(5)*2^30)
	for (i=40;i<60;i++) {

		f = (b & c) | (d & (b | c));
		e = leftrotate(a, 5) + f + e + k + w[i];
		b = leftrotate(b, 30);

		temp = e;
		e = d;
		d = c;
		c = b;
		b = a;
		a = temp;
	}

	k = 0xCA62C1D6;		// floor(sqrt(10)*2^30)
	for (i=60;i<80;i++) {

		f = b ^ c ^ d;
		e = leftrotate(a, 5) + f + e + k + w[i];
		b = leftrotate(b, 30);

		temp = e;
		e = d;
		d = c;
		c = b;
		b = a;
		a = temp;
	}

	h[0] += a;
	h[1] += b;
	h[2] += c;
	h[3] += d;
	h[4] += e;
}

/**
* \brief	SHA-1
*
* \param	chunk	Pointer to 64 byte chunks
* \param	count	Number of chunks
* \param	h	Pointer to current hash value
*/
void sha1updatechunks(const Uint8* chunk, Uint32 count, Uint32* h)
{
	while (count > 0) {
		sha1chunk(chunk, h);
		chunk += 64;
		count--;
	}
}
